#include <Arduino.h>

#include "IICtrans.h"

byte gIICData[4] = {0,0,0,0};
volatile byte gIICrecv = 0;

void IICread();
void IICwrite(byte len, word inAddr, byte inCmd, byte inDat, byte inStat=0);

IICTRANS::IICTRANS()
{
}

void IICTRANS::IICstart(byte inMode)
{
	if(inMode == LOW) { // CAB mode
		Wire.begin();
	}
	else
	{
		Wire.begin(8);
		Wire.onReceive(IICread);
	}
}

/*** IIC Recved ***/
byte IICTRANS::IICrecvByte()
{
	return gIICrecv;
}

/*** IIC Read data ***/
byte IICTRANS::getIICData(word AccOffset)
{
	if(gIICrecv == 0) return 0;
	
	byte Result = gIICrecv;
	byte pt;
	
	pt = 0;
	Addr = gIICData[pt++];
	if(gIICrecv == 4) {
		Addr += (gIICData[pt++] << 8);
	}
	
	//if((gLocProtcol == ADDR_MM2) && (IICAddr > MAX_MM_ADDR)) return 0;
	
	byte dat = gIICData[pt];
	byte cmd = dat & 0b11110000;
	
	switch(cmd) {
	  case 0:
		Cmd = IICCMD_NOP;
		Dir = 1;
		Dat = 0;
		break;
	  case 0b00100000: //Loco Speed
		Cmd = IICCMD_SPD;
		if((gIICrecv < 4) && (gIICData[++pt] == 255)) {
			Dir = 0;
			Dat = 255;
		} else {
			Dir = ((gIICData[pt] & 0x80) == 0) ? 2 : 1; // REV : FWD
			Dat = ((gIICData[pt] & 0x7f) << 1); // 127 -> 254
		}
		break;
		
	  case 0b01100000: //Loco Function
		Cmd = IICCMD_FN;
		Dat = dat & 0x0f | (gIICData[++pt] & 0x10);
		Stat = gIICData[pt] & 0x01;
		break;
		
	  case 0b01000000: //Trunout
		Cmd = IICCMD_ACC;
		Dat = gIICData[++pt];
		break;
		
	  case 0b00110000: // Consist address
		Cmd = IICCMD_CONS;
		Dat = gIICData[++pt];
		break;
		
	  case 0b01110000: // Ops mode
		Cmd = IICCMD_OPS;
		Dat = gIICData[++pt];
		break;
	}
	
	Serial.print("getIICData:");
	Serial.print(cmd);
	Serial.print(",");
	Serial.print(dat);
	Serial.print(",");
	Serial.print(Cmd);
	Serial.print(",");
	Serial.println(Dat);
	
	gIICrecv = 0;
	
	return Result;
}

// IIC read Interrupt
void IICread()
{
	gIICrecv = Wire.available();
	
	#ifdef _DEBUGPORT
	digitalWrite(PORT_LED, HIGH);
	#endif
	
	for(int i=0; i<gIICrecv; i++)
	{
		gIICData[i] = (byte)Wire.read();
	}
	
	#ifdef _DEBUGPORT
	digitalWrite(PORT_LED, LOW);
	#endif
}

/*** IIC Send data ***/
void IICTRANS::IICsend(word inAddr, byte inCmd, int inDat, byte inStat=0)
{
	byte dat;
	byte len = 3;
	if((inAddr != 255) && (inAddr > 127)) len = 4;
	
	switch(inCmd) {
	  case 0b00100000: // Loco Dir or Step
		IICwrite(len, inAddr, inCmd, (byte)inDat);
		break;
	  case 0b01100000: // Function
		dat = (byte)inDat;
		IICwrite(len, inAddr, inCmd | (dat & 0x0f), (dat & 0x10) | inStat);
		break;
	  case 0b01000000: // Accessory
		IICwrite(len, inAddr, inCmd, (byte)inDat);
		break;
	  case 0b00110000: // Consist address
		len = 4;
		IICwrite(len, inAddr, inCmd, (byte)inDat /*,gConsDir*/);
		break;
	  case 0b01110000: // Ops mode
		IICwrite(len, inAddr, inCmd, (byte)inDat, inStat);
		break;
	}
}

void IICwrite(byte len, word inAddr, byte inCmd, byte inDat, byte inStat=0)
{
	byte iicData[4] = {0,0,0,0};
	byte pt = 0;
	
	//if(inAddr < 128) { // primary address
	if(len == 3) { // primary address
		iicData[pt++] = inAddr;
	}
	else if(len == 4) {
		iicData[pt++] = inAddr % 256;
		iicData[pt++] = inAddr / 256;
	}
	
	iicData[pt++] = inCmd;
	iicData[pt++] = inDat;
	if(inCmd == 0b01110000) {
		iicData[pt++] = inStat;
	}
	
	Wire.beginTransmission(8);
	Wire.write(iicData, pt);
	Wire.endTransmission();
}
